# -*- coding: utf-8 -*-

import jinja2
import smtplib
import logging


def send_email(config, mail_from, rcpt_to, subject, template, vars={}):
    # settup email template
    template_loader = jinja2.FileSystemLoader(
        searchpath=template[0])
    template_env = jinja2.Environment(loader=template_loader)
    t = template_env.get_template(template[1])
    body = t.render(vars)

    # setup logging
    log = logging.getLogger("")

    # format msg
    message = "From: Access And Security <%s>\n" % mail_from
    message += "To: %s\n" % rcpt_to
    message += "Subject: %s\n\n" % subject
    message += body

    # send email
    log.debug("Sending email to %s with instructions" % rcpt_to)
    try:
        smtpObj = smtplib.SMTP(
            host=config['smtp_host'],
            port=config['smtp_port'],
            timeout=10
        )

        if (config['smtp_tls']):
            smtpObj.starttls()

        smtpObj.ehlo()

        if (config['smtp_auth']):
            smtpObj.login(
                config['smtp_user'],
                config['smtp_pass'])

        smtpObj.sendmail(mail_from, rcpt_to, message)
    except Exception as e:
        log.error("Error while sending email to %s: %s" % (rcpt_to, e))
        return False

    return True



# -*- coding: utf-8 -*-
"""ldapmgk

Usage:
    ldapmgk [options] user ACTION [<username>] [<email>] [<name>]
    ldapmgk [options] group ACTION [<groupname>] [<username>]
    ldapmgk [options] servergroup ACTION [<groupname>] [<username>]
    ldapmgk --version
    ldapmgk -h

User ACTION argument:
    list      List all users on the LDAP directory
    show      Show details for a user
    add       Add a new user to the LDAP directory
    remove    Delete a user from the LDAP directory
    resetpwd  Reset a user's password

Group ACTION argument:
    create    Add a new group to the LDAP directory
    delete    Delete a group from the LDAP directory
    add       Add a user to a group
    remove    Remove a user from a group

Server Group ACTION argument:
    add       Add a user to a server group
    remove    Remove a user from a server group

Examples:
    ldapmgk user list
    ldapmgk user add ldapmgk ldapmk@example.com "ldapmgk user"
    ldapmgk user show ldapmgk
    ldapmgk group create vpn
    ldapmgk group add vpn ldapmgk
    ldapmgk group show vpn
    ldapmgk servergroup add server_admin ldapmgk
    ldapmgk user resetpwd ldapmgk

Global Options:
    -c, --config=<file>     Configuration file [default:
                            /etc/ldapmgk/config.cfg]
    -o, --output=<file>     Log output to file
    -f, --force             Force some procedures
    -v, --verbose           Verbose output
    -d, --debug             Debug output
    -q, --quiet             Quiet output
    -h, --help              Show this screen
    -V, --version           Show version

"""
from inspect import getmembers, isclass
from docopt import docopt
import sys
import logging
from . import __version__ as VERSION


def main():
    """Main CLI entrypoint."""
    import commands

    options = docopt(
        __doc__,
        version="ldapmgk %s" % (VERSION)
    )

    # intialize logging
    if options['--debug'] is True:
        log_level = logging.DEBUG
    elif options['--verbose'] is True:
        log_level = logging.INFO
    else:
        log_level = logging.WARNING

    log = logging.getLogger("")
    log.setLevel(log_level)

    # stdout handler if needed
    if not (options['--quiet']):
        log_ch = logging.StreamHandler(sys.stdout)
        log_ch.setFormatter(logging.Formatter("%(levelname)s: %(message)s"))
        log.addHandler(log_ch)

    # file handler if needed
    if (options['--output']):
        log_fh = logging.FileHandler(options['--output'])
        log_fh.setFormatter(
            logging.Formatter("%(asctime)s [%(levelname)s] %(message)s"))
        log.addHandler(log_fh)

    # dinamically match the command line and load corresponding class
    for k, v in options.iteritems():
        if hasattr(commands, k) and v:
            module = getattr(commands, k)
            commands = getmembers(module, isclass)
            command = [
                command[1] for command in commands if command[0] != 'Base'
            ][0]
            command = command(options)
            command.run()



# -*- coding: utf-8 -*-
"""The group command class."""

from .base import Base
import ldap
import ldap.modlist as modlist
import pprint


class Group(Base):

    def run(self):
        try:
            action = self.options['ACTION'].lower()
        except:
            self.log.critical(
                "ldapmgk group: invalid action. It must be a string.")
            exit(1)

        groupname = self.options['<groupname>']

        if (self.options['<username>']):
            username = self.options['<username>']
        else:
            username = None

        if (action == 'list'):
            self.list_groups(echo=True)
        elif (action == 'show'):
            self.search(groupname, echo=True)
        elif (action == 'create'):
            if (self.add_group(groupname)):
                self.log.info("Sucessfully added group %s." % groupname)
            else:
                exit(1)

        elif (action == 'delete'):
            if (self.delete_group(groupname)):
                self.log.info("Sucessfully removed group %s." % groupname)
            else:
                exit(1)

        elif (action == 'add'):
            if (self.add_user_to_group(username, groupname)):
                self.log.info(
                    "Sucessfully added user %s to group %s." % (
                        username,
                        groupname,
                    ))
            else:
                exit(1)

        elif (action == 'remove'):
            if (self.remove_user_from_group(username, groupname)):
                self.log.info(
                    "Sucessfully removed user %s from group %s." % (
                        username,
                        groupname,
                    ))
            else:
                exit(1)

        else:
            self.log.critical(
                "ldapmgk group: unknown action '%s'." % action)
            exit(1)

        exit(0)

    def search(self, groupname, echo=False):
        self.log.debug("Searching for group.")
        group_filter = self._get_config_str('Main', 'LDAP_GROUP_FILTER')
        group_dn = self._get_config_str('Main', 'LDAP_GROUP_DN')
        filter = "(%s=%s)" % (group_filter, groupname)

        try:
            rs = self.ldap.search_s(
                group_dn,
                self.ldap_scope,
                filter
            )
        except ldap.INSUFFICIENT_ACCESS as e:
            self.permission_denied(e)

        if len(rs) > 0:
            if (echo):
                pp = pprint.PrettyPrinter(indent=2)
                pp.pprint(rs[0][1])
            else:
                return rs[0][0]
        else:
            if (echo):
                print "Group %s not found." % groupname

            return False

    def list_groups(self, echo=False):
        group_filter = self._get_config_str('Main', 'LDAP_GROUP_FILTER')
        group_dn = self._get_config_str('Main', 'LDAP_GROUP_DN')
        filter = "(%s=*)" % group_filter

        try:
            rs = self.ldap.search_s(
                group_dn,
                self.ldap_scope,
                filter,
                [group_filter]
            )
        except ldap.INSUFFICIENT_ACCESS as e:
            self.permission_denied(e)

        if (echo):
            print "Groups:"
            rs.sort(key=lambda i: i[1])
            for k, v in rs:
                print "  %s" % v[group_filter][0]
        else:
            return rs

    def add_group(self, groupname, description=None):
        group_dn = self.search(groupname)

        self.log.debug(
            "Adding group %s" % groupname)

        if (group_dn):
            self.log.warning("Group already exists. Nothing to do.")
            exit(1)
        else:
            groups_dn = self._get_config_str('Main', 'LDAP_GROUP_DN')
            group_dn = "cn=%s,%s" % (groupname, groups_dn)

        object_classes = self._get_config_str('Main', 'LDAP_GROUP_ATTRS')
        object_classes = object_classes.split(',')

        if not description:
            description = ''

        base_dn = self._get_config_str('Main', 'LDAP_BASE_DN')

        add_record = {
            'objectClass': object_classes,
            'cn': groupname,
            'description': description,
            'member': "cn=manager,%s" % base_dn
        }

        ldif = modlist.addModlist(add_record)

        try:
            self.ldap.add_s(group_dn, ldif)
        except ldap.LDAPError as e:
            self.log.error(
                "Error while adding group %s: %s" % (groupname, e))
            return False

        return True

    def delete_group(self, groupname):
        group_dn = self.search(groupname)

        self.log.debug(
            "Removing group %s" % groupname)

        if not (group_dn):
            self.log.warning("Group doesn't exist. Nothing to do.")
            exit(1)

        try:
            self.ldap.delete_s(group_dn)
        except ldap.LDAPError as e:
            self.log.error(
                "Error while removing group %s: %s" % (groupname, e))
            return False

        return True

    def add_user_to_group(self, username, groupname):
        group_dn = self.search(groupname)
        users_dn = self._get_config_str('Main', 'LDAP_USER_DN')
        user_dn = "cn=%s,%s" % (username, users_dn)

        if not (group_dn):
            self.log.warning("Group doesn't exist. Nothing to do.")
            exit(1)

        try:
            self.ldap.modify_s(
                group_dn,
                [(ldap.MOD_ADD, 'member', user_dn)]
            )
        except ldap.LDAPError as e:
            self.log.error(
                "Error while adding user %s to group %s: %s" % (
                    username,
                    groupname,
                    e))
            return False

        return True

    def remove_user_from_group(self, username, groupname):
        group_dn = self.search(groupname)
        users_dn = self._get_config_str('Main', 'LDAP_USER_DN')
        user_dn = "cn=%s,%s" % (username, users_dn)

        if not (group_dn):
            self.log.warning("Group doesn't exist. Nothing to do.")
            exit(1)

        try:
            self.ldap.modify_s(
                group_dn,
                [(ldap.MOD_DELETE, 'member', user_dn)]
            )
        except ldap.LDAPError as e:
            self.log.error(
                "Error while removing user %s from group %s: %s" % (
                    username,
                    groupname,
                    e))
            return False

        return True



# -*- coding: utf-8 -*-
"""The base command class, with shared options from all other commands."""


import ConfigParser
import logging
import ldap


class Base(object):
    """A base command."""

    def __init__(self, options, *args, **kwargs):
        self.options = options
        self.args = args
        self.kwargs = kwargs

        # load logging
        self.log = logging.getLogger("")

        # load configuration
        self.conf = ConfigParser.ConfigParser()
        self.conf.optionxform = str
        try:
            self.conf.read(self.options['--config'])
        except:
            self.log.warning(
                "Missing configuration from %s." % options['--config'])
            self.log.warning("Loading defaults from default-config.cfg.")
            try:
                self.conf.read('default-config.cfg')
            except:
                self.log.critical(
                   "Could not load configuration defaults. Exiting.")
                exit(1)

        # load ldap server
        ldap_server = "%s:%d" % (
            self._get_config_str('Main', 'LDAP_HOST'),
            self._get_config_int('Main', 'LDAP_PORT'))

        self.log.debug("Initializing LDAP Server %s" % ldap_server)
        try:
            self.ldap = ldap.initialize(ldap_server)
            self.ldap_scope = ldap.SCOPE_SUBTREE
        except ldap.LDAPError as e:
            self.log.critical(
                "Error initializing LDAP Server: %s" % e['desc'])
            exit(1)

        if (self._get_config_bool('Main', 'LDAP_TLS')):
            try:
                self.ldap.start_tls_s()
            except Exception as e:
                self.log.critical(
                    "Could not start TLS with the LDAP Server: %s." % e)
                exit(1)

        ldap_bind_dn = self._get_config_str('Main', 'LDAP_BIND_DN')
        ldap_bind_pw = self._get_config_str('Main', 'LDAP_BIND_PW')
        self.log.debug("Binding to the LDAP Server with '%s'." % ldap_bind_dn)
        try:
            self.ldap.simple_bind_s(ldap_bind_dn, ldap_bind_pw)
        except Exception as e:
            self.log.critical(
                "Error binding to LDAP Server: %s" % e)
            exit(1)

        # setup email configuration
        self.smtp = {}
        self.smtp['smtp_host'] = self._get_config_str(
            'Email', 'EMAIL_SMTP_HOST')
        self.smtp['smtp_port'] = self._get_config_int(
            'Email', 'EMAIL_SMTP_PORT')
        self.smtp['smtp_tls'] = self._get_config_bool(
            'Email', 'EMAIL_SMTP_TLS')
        self.smtp['smtp_auth'] = self._get_config_bool(
            'Email', 'EMAIL_SMTP_AUTH')
        self.smtp['smtp_user'] = self._get_config_str(
            'Email', 'EMAIL_SMTP_USER')
        self.smtp['smtp_pass'] = self._get_config_str(
            'Email', 'EMAIL_SMTP_PASS')

    def _get_config_bool(self, section, option):
        return self.conf.getboolean(section, option)

    def _get_config_int(self, section, option):
        return self.conf.getint(section, option)

    def _get_config_float(self, section, option):
        return self.conf.getfloat(section, option)

    def _get_config_str(self, section, option):
        return self.conf.get(section, option)

    def _get_config_items(self, section):
        return self.conf.items(section)

    def _get_config_all(self):
        list = []
        for s in self.conf.sections():
            new_list = self._get_config_items(s)
            list = list + new_list
        return list

    def permission_denied(self, e):
        self.log.error("Permission denied.")
        exit(2)

    def run(self):
        raise NotImplementedError(
            'You must implement the run() method yourself!')



# -*- coding: utf-8 -*-
"""The user command class."""

import ldap
import ldap.modlist as modlist
import datetime
import pprint

from .base import Base
from ldapmgk.utils import generate_password
from ldapmgk.email import send_email


class User(Base):

    def run(self):
        # determine if using force parameter
        if (self.options['--force']):
            self.force = True
        else:
            self.force = False

        # determine if we'll send emails
        self.email_user_add = self._get_config_bool('Email', 'EMAIL_USER_ADD')
        self.email_user_resetpwd = self._get_config_bool(
             'Email', 'EMAIL_USER_RESETPWD')

        try:
            action = self.options['ACTION'].lower()
        except:
            self.log.critical(
                "ldapmgk user: invalid action. It must be a string.")
            exit(1)

        username = self.options['<username>']
        email = self.options['<email>']
        name = self.options['<name>']

        if (action == 'list'):
            self.list_users(echo=True)
        elif (action == 'show'):
            self.search(username, echo=True)
        elif (action == 'add'):
            if (name):
                name = name.split(' ')
            else:
                name = username.split('.')

            # since name is invalid, let's improvise
            if len(name) < 2:
                name = [username, username]

            password = self.add_user(username, email, name)
            if (password):
                self.log.info(
                    "Sucessfully added user %s with temporary password: %s" % (
                        username, password))
            else:
                exit(1)

        elif (action == 'remove'):
            if (self.delete_user(username)):
                self.log.info("Sucessfully removed user %s." % username)
            else:
                exit(1)

        elif (action == 'resetpwd'):
            password = self.reset_pwd(username, email)
            if (password):
                self.log.info(
                    "Sucessfully reset user %s password " % username +
                    "with temporary password: %s" % password)
            else:
                exit(1)

        else:
            self.log.critical(
                "ldapmgk user: unknown action '%s'." % action)
            exit(1)

        exit(0)

    def search(self, username, echo=False):
        self.log.debug("Searching for user.")
        user_filter = self._get_config_str('Main', 'LDAP_USER_FILTER')
        user_dn = self._get_config_str('Main', 'LDAP_USER_DN')
        filter = "(%s=%s)" % (user_filter, username)

        try:
            rs = self.ldap.search_s(
                user_dn,
                self.ldap_scope,
                filter
            )
        except ldap.INSUFFICIENT_ACCESS as e:
            self.permission_denied(e)

        # search in the inactive users tree if needed
        move_user = self._get_config_bool('Main', 'LDAP_MOVE_USER_ON_DEL')
        if (move_user):
            move_user_dn = self._get_config_str('Main', 'LDAP_MOVE_USER_DN')
            try:
                move_rs = self.ldap.search_s(
                    move_user_dn,
                    self.ldap_scope,
                    filter,
                    [user_filter]
                )
            except ldap.INSUFFICIENT_ACCESS as e:
                self.permission_denied(e)

        else:
            move_rs = None

        rs = rs + move_rs

        if len(rs) > 0:
            if (echo):
                pp = pprint.PrettyPrinter(indent=2)
                pp.pprint(rs[0][1])
            else:
                return rs[0][0]
        else:
            if (echo):
                print "User %s not found." % username

            return False

    def get_next_uid(self):
        self.log.debug("Searching for the next UID.")
        user_dn = self._get_config_str('Main', 'LDAP_USER_DN')

        try:
            rs = self.ldap.search_s(
                user_dn,
                self.ldap_scope,
                "(uidNumber=*)",
                ['uidNumber']
            )
        except ldap.INSUFFICIENT_ACCESS as e:
            self.permission_denied(e)

        # search in the inactive users tree if needed
        move_user = self._get_config_bool('Main', 'LDAP_MOVE_USER_ON_DEL')
        if (move_user):
            move_user_dn = self._get_config_str('Main', 'LDAP_MOVE_USER_DN')
            try:
                move_rs = self.ldap.search_s(
                    move_user_dn,
                    self.ldap_scope,
                    "(uidNumber=*)",
                    ['uidNumber']
                )
            except ldap.INSUFFICIENT_ACCESS as e:
                self.permission_denied(e)

            rs = rs + move_rs
        else:
            move_rs = None

        rs.sort(reverse=True, key=lambda i: i[1])
        try:
            last_uid = rs[0][1]['uidNumber'][0]
        except Exception as e:
            self.log.error("Couldn't find last UID. Error: %s" % e)

        self.log.debug("Found last UID used: %s" % last_uid)

        next_uid = int(last_uid) + 1
        self.log.debug("Returning next unused UID: %d" % next_uid)
        return next_uid

    def list_users(self, echo=False):
        user_filter = self._get_config_str('Main', 'LDAP_USER_FILTER')
        user_dn = self._get_config_str('Main', 'LDAP_USER_DN')
        filter = "(%s=*)" % user_filter

        try:
            act_rs = self.ldap.search_s(
                user_dn,
                self.ldap_scope,
                filter,
                [user_filter]
            )
        except ldap.INSUFFICIENT_ACCESS as e:
            self.permission_denied(e)

        # search in the inactive users tree if needed
        inact_user = self._get_config_bool('Main', 'LDAP_MOVE_USER_ON_DEL')
        if (inact_user):
            inact_user_dn = self._get_config_str('Main', 'LDAP_MOVE_USER_DN')
            try:
                inact_rs = self.ldap.search_s(
                    inact_user_dn,
                    self.ldap_scope,
                    filter,
                    [user_filter]
                )
            except ldap.INSUFFICIENT_ACCESS as e:
                self.permission_denied(e)

        else:
            inact_rs = None

        rs = {
            'active': act_rs,
            'inactive': inact_rs
        }

        if (echo):
            print "Active users:"
            rs['active'].sort(key=lambda i: i[1])
            for k, v in rs['active']:
                print "  %s" % v[user_filter][0]
            print

            print "Inactive users:"
            rs['inactive'].sort(key=lambda i: i[1])
            for k, v in rs['inactive']:
                print "  %s" % v[user_filter][0]
        else:
            return rs

    def add_user(self, username, email, name):

        user_dn = self.search(username)
        if (user_dn):
            self.log.warning("User already exists. Nothing to do.")
            exit(1)
        else:
            users_dn = self._get_config_str('Main', 'LDAP_USER_DN')
            user_dn = "cn=%s,%s" % (username, users_dn)

        full_name = "%s %s" % (name[0], name[1])

        self.log.debug(
            "Adding user %s (%s) - %s" % (username, email, full_name))

        object_classes = self._get_config_str('Main', 'LDAP_USER_ATTRS')
        object_classes = object_classes.split(',')

        # generate a random password
        wordlist_file = self._get_config_str(
            'Main', 'LDAP_USER_PW_RANDOM_FILE')
        with open(wordlist_file) as f:
            words = [w.strip() for w in f]
        password = generate_password(
            words,
            numbers='0123456789',
            characters='!@#$%'
        )

        add_record = {
            'objectClass': object_classes,
            'cn': username,
            'uid': username,
            'uidNumber': str(self.get_next_uid()),
            'gidNumber': '1200',
            'loginShell': '/bin/bash',
            'homeDirectory': "/home/%s" % username,
            'mail': email,
            'userPassword': password,
            'givenName': name[0],
            'sn': name[1],
            'displayName': "%s %s" % (name[0], name[1]),
        }

        ldif = modlist.addModlist(add_record)

        try:
            self.ldap.add_s(user_dn, ldif)
        except ldap.LDAPError as e:
            self.log.error(
                "Error while adding user %s: %s" % (username, e))
            return False

        # use expire rules control if needed
        expire = self._get_config_bool('Main', 'LDAP_USER_ADD_EXPIRE')
        if (expire):
            y = datetime.datetime.now() - datetime.timedelta(days=1*365)
            past = y.strftime('%Y%m%d%H%M%SZ')
            serverctrls = [
                ldap.controls.simple.RelaxRulesControl()
            ]

            try:
                self.ldap.modify_ext_s(
                    user_dn,
                    [(ldap.MOD_REPLACE, 'pwdChangedTime', past)],
                    serverctrls)
            except ldap.LDAPError as e:
                self.log.error(
                     "Error while changing pwdChangedTime on user %s: %s" % (
                         username,
                         e))
                return False

        # send email if needed
        if (self.email_user_add):
            mail_from = self._get_config_str('Email', 'EMAIL_FROM')
            template_dir = self._get_config_str('Email', 'EMAIL_TEMPLATE_DIR')
            template_file = self._get_config_str('Email', 'EMAIL_TEMPLATE')
            subject = self._get_config_str('Email', 'EMAIL_SUBJECT_PREFIX')
            subject += " New access created"
            template = (template_dir, template_file)
            vars = {
                'username': username,
                'password': password
            }

            send_email(self.smtp, mail_from, email, subject, template, vars)

        return password

    def delete_user(self, username):
        user_dn = self.search(username)

        self.log.debug(
            "Removing user %s" % username)

        if not (user_dn):
            self.log.warning("User doesn't exist. Nothing to do.")
            exit(1)

        move_user = self._get_config_bool('Main', 'LDAP_MOVE_USER_ON_DEL')
        if (move_user and not self.force):
            move_user_dn = self._get_config_str('Main', 'LDAP_MOVE_USER_DN')
            new_user_dn = "cn=%s" % username
            try:
                self.ldap.rename_s(
                    user_dn,
                    new_user_dn,
                    move_user_dn,
                    True
                )
            except ldap.LDAPError as e:
                self.log.error(
                    "Error while removing (moving to inactive) user %s: %s" % (
                        username,
                        e
                    )
                )
                return False
        else:
            try:
                self.ldap.delete_s(user_dn)
            except ldap.LDAPError as e:
                self.log.error(
                    "Error while removing user %s: %s" % (username, e))
                return False

        return True

    def reset_pwd(self, username, email):

        user_dn = self.search(username)

        self.log.debug(
            "Resetting password for user %s" % username)

        if not (user_dn):
            self.log.warning("User doesn't exist. Nothing to do.")
            exit(1)

        # generate a random password
        wordlist_file = self._get_config_str(
            'Main', 'LDAP_USER_PW_RANDOM_FILE')
        with open(wordlist_file) as f:
            words = [w.strip() for w in f]
        password = generate_password(
            words,
            numbers='0123456789',
            characters='!@#$%'
        )

        try:
            self.ldap.modify_s(
                user_dn,
                [(ldap.MOD_REPLACE, 'userPassword', password)]
            )
        except ldap.LDAPError as e:
            self.log.error(
                "Error while resetting password for user %s: %s" % (
                    username,
                    e))
            return False

        # use expire rules control if needed
        expire = self._get_config_bool('Main', 'LDAP_USER_ADD_EXPIRE')
        if (expire):
            y = datetime.datetime.now() - datetime.timedelta(days=self._get_config_int('Main', 'LDAP_USER_ADD_EXPIRE_TIME'))
            past = y.strftime('%Y%m%d%H%M%SZ')
            serverctrls = [
                ldap.controls.simple.RelaxRulesControl()
            ]

            try:
                self.ldap.modify_ext_s(
                    user_dn,
                    [(ldap.MOD_REPLACE, 'pwdChangedTime', past)],
                    serverctrls)
            except ldap.LDAPError as e:
                self.log.error(
                     "Error while changing pwdChangedTime on user %s: %s" % (
                         username,
                         e))
                return False

        # send email if needed
        if (self.email_user_resetpwd):
            mail_from = self._get_config_str('Email', 'EMAIL_FROM')
            template_dir = self._get_config_str('Email', 'EMAIL_TEMPLATE_DIR')
            template_file = self._get_config_str('Email', 'EMAIL_TEMPLATE')
            subject = self._get_config_str('Email', 'EMAIL_SUBJECT_PREFIX')
            subject += " Password reset"
            template = (template_dir, template_file)
            vars = {
                'username': username,
                'password': password
            }

            send_email(self.smtp, mail_from, email, subject, template, vars)

        return password



# -*- coding: utf-8 -*-

import random

r = random.SystemRandom()


def generate_password(words, top=2000, k=4, numbers=None, characters=None,
                      first_upper=True):
    """Return a random password based on a sorted word list."""
    elements = r.sample(words[:top], k)

    if numbers:
        elements.insert(r.randint(1, len(elements)), r.choice(numbers))
    if characters:
        elements.insert(r.randint(1, len(elements)), r.choice(characters))
    if first_upper:
        elements[0] = elements[0].title()

    return ''.join(elements)



# -*- coding: utf-8 -*-
"""The servergroup command class."""

from .base import Base
import ldap
import pprint


class Group(Base):

    def run(self):
        try:
            action = self.options['ACTION'].lower()
        except:
            self.log.critical(
                "ldapmgk servergroup: invalid action. It must be a string.")
            exit(1)

        groupname = self.options['<groupname>']
        username = self.options['<username>']

        if (action == 'list'):
            self.list_servergroups(echo=True)
        elif (action == 'show'):
            self.search(groupname, echo=True)
        elif (action == 'add'):
            if (self.add_user_to_servergroup(username, groupname)):
                self.log.info(
                    "Sucessfully added user %s to server group %s." % (
                        username,
                        groupname,
                    ))
            else:
                exit(1)

        elif (action == 'remove'):
            if (self.remove_user_from_servergroup(username, groupname)):
                self.log.info(
                    "Sucessfully removed user %s from server group %s." % (
                        username,
                        groupname,
                    ))
            else:
                exit(1)

        else:
            self.log.critical(
                "ldapmgk group: unknown action '%s'." % action)
            exit(1)

        exit(0)

    def search(self, groupname, echo=False):
        self.log.debug("Searching for server group.")
        group_filter = self._get_config_str('Main', 'LDAP_SERVER_GROUP_FILTER')
        group_dn = self._get_config_str('Main', 'LDAP_SERVER_GROUP_DN')
        filter = "(%s=%s)" % (group_filter, groupname)

        try:
            rs = self.ldap.search_s(
                group_dn,
                self.ldap_scope,
                filter
            )
        except ldap.INSUFFICIENT_ACCESS as e:
            self.permission_denied(e)

        if len(rs) > 0:
            if (echo):
                pp = pprint.PrettyPrinter(indent=2)
                pp.pprint(rs[0][1])
            else:
                return rs[0][0]
        else:
            if (echo):
                print "Server Group %s not found." % groupname

            return False

    def list_servergroups(self, echo=False):
        group_filter = self._get_config_str('Main', 'LDAP_SERVER_GROUP_FILTER')
        group_dn = self._get_config_str('Main', 'LDAP_SERVER_GROUP_DN')
        filter = "(%s=*)" % group_filter

        try:
            rs = self.ldap.search_s(
                group_dn,
                self.ldap_scope,
                filter,
                [group_filter]
            )
        except ldap.INSUFFICIENT_ACCESS as e:
            self.permission_denied(e)

        if (echo):
            print "Server Groups:"
            rs.sort(key=lambda i: i[1])
            for k, v in rs:
                print "  %s" % v[group_filter][0]
        else:
            return rs

    def add_user_to_servergroup(self, username, groupname):
        group_dn = self.search(groupname)

        if not (group_dn):
            self.log.warning("Server group doesn't exist. Nothing to do.")
            exit(0)

        try:
            self.ldap.modify_s(
                group_dn,
                [(ldap.MOD_ADD, 'memberUid', username)]
            )
        except ldap.LDAPError as e:
            self.log.error(
                "Error while adding user %s to server group %s: %s" % (
                    username,
                    groupname,
                    e))
            return False

        return True

    def remove_user_from_servergroup(self, username, groupname):
        group_dn = self.search(groupname)

        if not (group_dn):
            self.log.warning("Group doesn't exist. Nothing to do.")
            exit(0)

        try:
            self.ldap.modify_s(
                group_dn,
                [(ldap.MOD_DELETE, 'memberUid', username)]
            )
        except ldap.LDAPError as e:
            self.log.error(
                "Error while removing user %s from server group %s: %s" % (
                    username,
                    groupname,
                    e))
            return False

        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Packaging settings"""

import io
from ldapmgk import __version__
from os.path import abspath, dirname, join

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

this_dir = abspath(dirname(__file__))
with io.open(join(this_dir, 'README.rst'), encoding='utf-8') as file:
    long_description = file.read()

config = {
    'name': 'ldapmgk',
    'version': __version__,
    'description': 'Helper for managing an LDAP directory on CLI',
    'long_description': long_description,
    'url': 'https://github.com/Movile/ldapmgk',
    'author': 'Hugo Cisneiros',
    'author_email': 'hugo.cisneiros@movile.com',
    'license': 'GPLv3',
    'classifiers': [
        'Intended Audience :: System Administrators',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ],
    'keywords': [
        'ldap',
        'manager'
    ],
    'packages': find_packages(exclude=['docs', 'tests*']),
    'install_requires': ['docopt', 'python-ldap', 'Jinja2'],
    'extras_require': {
        'test': ['nose'],
    },
    'entry_points': {
        'console_scripts': [
            'ldapmgk=ldapmgk.cli:main',
        ],
    },
}

setup(**config)
